# Generated by Django 2.2.9 on 2020-12-04 11:18

import json

from django.db import migrations
from invoicing.models import Invoice


def string_supplier_additional_info(*args, **kwargs):
    for invoice in Invoice.objects.all().only('id', 'supplier_additional_info'):
        if isinstance(invoice.supplier_additional_info, dict):
            invoice.supplier_additional_info = json.dumps(invoice.supplier_additional_info)
            invoice.save(update_fields=['supplier_additional_info'])


def json_supplier_additional_info(*args, **kwargs):
    for invoice in Invoice.objects.all().only('id', 'supplier_additional_info'):
        if invoice.supplier_additional_info is None:
            invoice.supplier_additional_info = {}
            invoice.save(update_fields=['supplier_additional_info'])

        if isinstance(invoice.supplier_additional_info, str):
            invoice.supplier_additional_info = json.loads(invoice.supplier_additional_info)
            invoice.save(update_fields=['supplier_additional_info'])


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0024_auto_20200901_1241'),
    ]

    operations = [
        migrations.RunPython(json_supplier_additional_info, string_supplier_additional_info)

    ]



from django.conf import settings
from django.conf.urls import patterns, include, url

# Uncomment the next two lines to enable the admin:
from django.contrib import admin


admin.autodiscover()

urlpatterns = patterns('',
    # Uncomment the admin/doc line below to enable admin documentation:
    url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    url(r'^admin/', include(admin.site.urls)),
)


if 'invoicing' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'^invoicing/', include('invoicing.urls', app_name='invoicing', namespace='invoicing')),
    )



from django.db import migrations, transaction
from invoicing.models import Invoice


def recalculate(*args, **kwargs):
    pass
    # try:
    #     for invoice in Invoice.objects.all():
    #         with transaction.atomic():
    #             invoice.total = invoice.calculate_total()
    #             invoice.vat = invoice.calculate_vat()
    #             invoice.save(update_fields=['total'])
    #             invoice.save(update_fields=['vat'])
    # except:
    #     pass


class Migration(migrations.Migration):
    dependencies = [
        ('invoicing', '0013_rename_full_number_to_number'),
    ]

    operations = [
        migrations.RunPython(recalculate, lambda *args, **kwargs: None)
    ]



# Generated by Django 2.0.6 on 2018-11-21 08:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0018_invoice_attachments'),
        # ('invoicing', '0020_auto_20181001_1025'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='related_document',
            field=models.CharField(blank=True, max_length=100),
        ),
    ]



# Generated by Django 2.0.6 on 2018-06-19 11:49

from django.db import migrations, models
import django_countries.fields
from internationalflavor.vat_number import VATNumberField


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0016_update_credit_notes'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='customer_email',
            field=models.EmailField(blank=True, max_length=254),
        ),
        migrations.AddField(
            model_name='invoice',
            name='customer_phone',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='bank_city',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='bank_country',
            field=django_countries.fields.CountryField(blank=True, default='', max_length=2),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='bank_name',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='bank_street',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='bank_zip',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='constant_symbol',
            field=models.CharField(blank=True, default='', max_length=64),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='customer_city',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='customer_registration_id',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='customer_street',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='customer_tax_id',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='customer_vat_id',
            field=VATNumberField(blank=True, default='', max_length=13),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='customer_zip',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='delivery_method',
            field=models.CharField(default='PERSONAL_PICKUP', max_length=64),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='issuer_email',
            field=models.EmailField(blank=True, default='', max_length=254),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='issuer_name',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='issuer_phone',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='note',
            field=models.CharField(blank=True, default='Thank you for using our services.', max_length=255),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='reference',
            field=models.CharField(blank=True, default='', max_length=140),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='shipping_city',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='shipping_country',
            field=django_countries.fields.CountryField(blank=True, default='', max_length=2),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='shipping_name',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='shipping_street',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='shipping_zip',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='status',
            field=models.CharField(default='NEW', max_length=64),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='subtitle',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='supplier_city',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='supplier_registration_id',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='supplier_street',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='supplier_tax_id',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='supplier_vat_id',
            field=VATNumberField(blank=True, default='', max_length=13),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='supplier_zip',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='type',
            field=models.CharField(default='INVOICE', max_length=64),
        ),
        migrations.AlterField(
            model_name='item',
            name='unit',
            field=models.CharField(default='PIECES', max_length=64),
        ),
    ]



from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from invoicing.models import Item


@receiver(post_save, sender=Item)
@receiver(post_delete, sender=Item)
def recalculate(instance, **kwargs):
    invoice = instance.invoice
    invoice.total = invoice.calculate_total()
    invoice.vat = invoice.calculate_vat()
    invoice.save(update_fields=['total', 'vat'])



# Generated by Django 3.1.12 on 2021-06-22 08:55

from django.db import migrations

try:
    # Django 3.1
    from django.db.models import JSONField
except ImportError:
    # older Django
    from django.contrib.postgres.fields import JSONField


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0025_supplier_additional_info_json'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invoice',
            name='attachments',
            field=JSONField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='supplier_additional_info',
            field=JSONField(blank=True, default=None, null=True),
        ),
    ]



# Django settings for example project.
from os.path import normpath, join
from django.core.urlresolvers import reverse_lazy


DEBUG = True
TEMPLATE_DEBUG = DEBUG

LOGOUT_REDIRECT_URL = reverse_lazy('login')
LOGIN_URL = reverse_lazy('login')

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',  # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': 'example',                      # Or path to database file if using sqlite3.
        'USER': 'example',                      # Not used with sqlite3.
        'PASSWORD': 'example',                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
}

# Hosts/domain names that are valid for this site; required if DEBUG is False
# See https://docs.djangoproject.com/en/1.4/ref/settings/#allowed-hosts
ALLOWED_HOSTS = []

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = ''

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = ''

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'v_r3_%18%ep5wj!$k67fe73+jpr7xdnh=2#6j#o1&amp;*q8o2y#yg'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    # Uncomment the next line for simple clickjacking protection:
    # 'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'example.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'example.wsgi.application'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # Uncomment the next line to enable the admin:
    'django.contrib.admin',
    # Uncomment the next line to enable admin documentation:
    'django.contrib.admindocs',

    'south',

    'example',
    'invoicing',
)

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



# INVOICING

# Note: the value None means "TAX not applicable" (supplier is not a VAT payer),
# rather than value Decimal(0) which means 0% TAX.
from decimal import Decimal
INVOICING_TAX_RATE = Decimal(20)

import json
INVOICING_SUPPLIER = {
    'name': 'Example company',
    'street': 'Example street',
    'city': 'Example city',
    'zip': 'Example ZIP code',
    'country_code': 'SK',
    'registration_id': '123 456 789',
    'tax_id': '111222333',
    'vat_id': 'SK111222333',
    'additional_info': json.dumps({
        "www": "www.example.com"
        #... legal matters
    }),
    'bank': {
        'name': 'Example bank',
        'street': 'Example street',
        'zip': 'Example ZIP code',
        'city': 'Example city',
        'country_code': 'SK',
        'iban': 'SK0000000000000000000028',
        'swift_bic': 'EXAMPLEBANK'
    }
}
# TODO: this setting is not ready yet
# Invoices are generated on the fly from database records.
# Therefore changing this value will affect all previously created invoices.
INVOICING_FORMATTER = 'invoicing.formatters.html.BootstrapHTMLFormatter'

INVOICING_SUPPLIER_LOGO_URL = normpath(join(STATIC_URL, 'my_logo.png'))

from invoicing.models import Invoice
INVOICING_COUNTER_PERIOD = Invoice.COUNTER_PERIOD.MONTHLY

# Remember to set INVOICING_NUMBER_FORMAT manually to match preferred way of invoice numbering schema.
# For example if you choose reset counter on daily basis, you need to use in INVOICING_NUMBER_FORMAT
# at least {{ invoice.date_issue|date:'d/m/Y' }} to distinguish invoice's full numbers between days.
INVOICING_NUMBER_FORMAT = "{{ invoice.date_issue|date:'Y/m' }}/{{ invoice.number }}"



from django.db import migrations, transaction
from invoicing.models import Invoice


def recalculate(*args, **kwargs):
    for invoice in Invoice.objects.all().only('total', 'vat'):
        with transaction.atomic():
            invoice.total = invoice.calculate_total()
            invoice.vat = invoice.calculate_vat()
            invoice.save(update_fields=['total'])
            invoice.save(update_fields=['vat'])


class Migration(migrations.Migration):
    dependencies = [
        ('invoicing', '0018_invoice_attachments'),
        ('invoicing', '0021_invoice_related_document'),
    ]

    operations = [
        migrations.RunPython(recalculate, lambda *args, **kwargs: None)
    ]



from django.contrib import admin, messages
from django.db.models import F
from django.db.models.functions import Coalesce
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from invoicing.managers import get_accounting_software_manager
from invoicing.models import Invoice, Item


class ItemInline(admin.TabularInline):
    fieldsets = (
        (
            None,
            {
                'fields': ('title', 'quantity', 'unit', 'unit_price', 'discount', 'tax_rate', 'weight')
            }
        ),
    )
    model = Item
    extra = 0


class OverdueFilter(admin.SimpleListFilter):
    title = _('overdue')
    parameter_name = 'overdue'

    def lookups(self, request, model_admin):
        return (
            ('no', _('no')),
            ('yes', _('yes')),
        )

    def queryset(self, request, queryset):
        if self.value() == 'no':
            return queryset.not_overdue()
        if self.value() == 'yes':
            return queryset.overdue()


@admin.register(Invoice)
class InvoiceAdmin(admin.ModelAdmin):
    date_hierarchy = 'date_issue'
    ordering = ['-date_issue', '-sequence']
    actions = ['send_to_accounting_software', 'export']
    list_display = ['pk', 'type', 'number', 'status',
                    'supplier_info', 'customer_info',
                    'annotated_subtotal', 'vat', 'total',
                    'currency', 'date_issue', 'payment_term_days', 'is_overdue_boolean', 'is_paid']
    list_editable = ['status']
    list_filter = ['type', 'status', 'payment_method', 'delivery_method', OverdueFilter, 'language', 'currency']
    search_fields = ['number', 'subtitle', 'note', 'supplier_name', 'customer_name', 'shipping_name']
    inlines = (ItemInline, )
    autocomplete_fields = ('related_invoices',)
    fieldsets = (
        (_(u'General information'), {
            'fields': (
                'type', 'status', 'language', ('sequence', 'number'), 'subtitle', 'related_document', 'related_invoices',
                'date_issue', 'date_tax_point', 'date_due', 'date_sent', 'date_paid',
                'note'
            )
        }),
        (_(u'Contact details'), {
            'fields': (
                'issuer_name', 'issuer_email', 'issuer_phone'
            )
        }),
        (_(u'Payment details'), {
            'fields': (
                'currency', 'credit', #'already_paid',
                ('payment_method', 'delivery_method'),
                ('constant_symbol', 'variable_symbol', 'specific_symbol', 'reference'),
                'bank_name', 'bank_country', 'bank_city', 'bank_street', 'bank_zip', 'bank_iban', 'bank_swift_bic'
            )
        }),
        (_(u'Supplier details'), {
            'fields': (
                'supplier_name', 'supplier_street', 'supplier_zip', 'supplier_city', 'supplier_country',
                'supplier_registration_id', 'supplier_tax_id', 'supplier_vat_id', 'supplier_additional_info'

            )
        }),
        (_(u'Customer details'), {
            'fields': (
                'customer_name', 'customer_street', 'customer_zip', 'customer_city', 'customer_country',
                'customer_registration_id', 'customer_tax_id', 'customer_vat_id', 'customer_additional_info',
            )
        }),
        (_(u'Shipping details'), {
            'fields': (
                'shipping_name', 'shipping_street', 'shipping_zip', 'shipping_city', 'shipping_country'
            )
        })
    )

    def get_queryset(self, request):
        return self.model.objects.annotate(annotated_subtotal=F('total')-Coalesce(F('vat'), 0))

    def annotated_subtotal(self, invoice):
        return invoice.annotated_subtotal
    annotated_subtotal.short_description = _(u'subtotal')

    def supplier_info(self, invoice):
        return mark_safe(u'%s<br>%s' % (invoice.supplier_name, invoice.supplier_country.name))
    supplier_info.short_description = _(u'supplier')

    def customer_info(self, invoice):
        return mark_safe(u'%s<br>%s' % (invoice.customer_name, invoice.customer_country.name))
    customer_info.short_description = _(u'customer')

    def payment_term_days(self, invoice):
        return u'%s days' % invoice.payment_term
    payment_term_days.short_description = _(u'payment term')

    def is_overdue_boolean(self, invoice):
        return invoice.is_overdue
    is_overdue_boolean.boolean = True
    is_overdue_boolean.short_description = _(u'is overdue')

    def is_paid(self, invoice):
        return invoice.status == Invoice.STATUS.PAID
    is_paid.boolean = True
    is_paid.short_description = _(u'is paid')

    def send_to_accounting_software(self, request, queryset):
        manager = get_accounting_software_manager()

        if manager is None:
            messages.error(request, _('Missing specification of accounting software'))
            return

        try:
            result = manager.send_to_accounting_software(request, queryset)

            if isinstance(result, str):
                messages.success(request, result)
            elif isinstance(result, list):
                success = 0

                for r in result:
                    if r['status_code'] == 200:
                        success += 1
                    else:
                        messages.error(request, f"[{r['invoice']}]: {r['status_code']} ({r['reason']})")

                if success > 0:
                    messages.success(request, _('%d invoices sent to accounting software') % success)

        except Exception as e:
            messages.error(request, e)

    send_to_accounting_software.short_description = _('Send to accounting software')

    def export(self, request, queryset):
        from invoicing.exporters import InvoiceXlsxListExporter

        # init exporter
        exporter = InvoiceXlsxListExporter(
            user=request.user,
            recipients=[request.user],
            selected_fields=None
        )
        exporter.queryset = queryset

        # export to file
        return exporter.export_to_response()

    export.short_description = _('Export to xlsx')



from django.db import migrations
from invoicing.models import Invoice


def recalculate_vat(*args, **kwargs):
    pass
    # try:
    #     for invoice in Invoice.objects.all():
    #         invoice.vat = invoice.calculate_vat()
    #         invoice.save(update_fields=['vat'])
    # except:
    #     pass


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0009_auto_20180531_1816'),
    ]

    operations = [
        migrations.RunPython(recalculate_vat, lambda *args, **kwargs: None)
    ]



from collections import OrderedDict

from django.utils.translation import ugettext_lazy as _, ugettext

from outputs.mixins import ExcelExporterMixin
from invoicing.models import Invoice


class InvoiceXlsxListExporter(ExcelExporterMixin):
    queryset = None
    filename = _('invoices.xlsx')

    @staticmethod
    def selectable_fields():
        # attribute, label, width, format (self.FORMATS), value
        return OrderedDict({
            ugettext('Details'): [
                ('id', ugettext('ID'), 7, 'integer'),
                ('created', ugettext('Created'), 20, 'datetime'),
                ('get_type_display', ugettext('Type'), 10, None, lambda value: value()),
                ('sequence', ugettext('Sequence'), 10),
                ('number', ugettext('Number'), 15),
                ('get_status_display', ugettext('Status'), 10, None, lambda value: value()),
                ('subtitle', ugettext('Subtitle'), 20),
                ('get_language_display', ugettext('Language'), 10, None, lambda value: value()),
                ('note', ugettext('Note'), 30),
            ],
            ugettext('Dates'): [
                ('date_issue', ugettext('Issue date'), 15, 'date'),
                ('date_tax_point', ugettext('Tax point date'), 15, 'date'),
                ('date_due', ugettext('Due date'), 15, 'date'),
                ('date_sent', ugettext('Sent (date)'), 15, 'date'),
                ('date_paid', ugettext('Date of payment'), 15, 'date', lambda value: value.date() if value else None),
                ('payment_term', ugettext('Payment term (days)'), 15, 'integer'),
                ('overdue_days', ugettext('Overdue (days)'), 15, 'integer', lambda value, obj: (
                    '=IF(TODAY()<=%(date_due)d, "", _xlfn.DAYS(TODAY(),%(date_due)d))' % {
                        'date_due': ExcelExporterMixin.to_excel_datetime(obj.date_due)
                    } if obj.status not in [Invoice.STATUS.PAID, Invoice.STATUS.CANCELED] else '', obj.overdue_days if obj.is_overdue else '')),
            ],
            ugettext('Payment'): [
                ('total', ugettext('Total'), 10),
                ('vat', ugettext('VAT'), 10),
                ('get_currency_display', ugettext('Currency'), 10, None, lambda value: value()),
                ('credit', ugettext('Credit'), 10),
                ('get_payment_method_display', ugettext('Payment method'), 20, None, lambda value: value()),
                ('constant_symbol', ugettext('Constant symbol'), 20),
                ('variable_symbol', ugettext('Variable symbol'), 20),
                ('specific_symbol', ugettext('Specific symbol'), 20),
                ('reference', ugettext('Reference'), 20),
                ('bank_name', ugettext('Bank name'), 20),
                # ('bank_street', ugettext('Bank street'), 20),
                # ('bank_zip', ugettext('Bank zip'), 15),
                # ('bank_city', ugettext('Bank city'), 20),
                # ('get_bank_country_display', ugettext('Bank country'), 20, None, lambda value: value()),
                ('bank_iban', ugettext('IBAN'), 30),
                ('bank_swift_bic', ugettext('SWIFT/BIC'), 15),
            ],
            ugettext('Issuer'): [
                ('supplier_name', ugettext('Supplier name'), 20),
                ('supplier_street', ugettext('Supplier street'), 20),
                ('supplier_zip', ugettext('Supplier zip'), 15),
                ('supplier_city', ugettext('Supplier city'), 15),
                ('get_supplier_country_display', ugettext('Supplier country'), 20, None, lambda value: value()),
                ('supplier_registration_id', ugettext('Supplier reg. ID'), 20),
                ('supplier_tax_id', ugettext('Supplier tax ID'), 20),
                ('supplier_vat_id', ugettext('Supplier VAT ID'), 20),
                ('supplier_additional_info', ugettext('Supplier additional info'), 25, None, lambda ord_dict: ', '.join(': '.join([str(label), str(value)]) for label, value in ord_dict.items()) if ord_dict and not isinstance(ord_dict, str) else ''),
                ('issuer_name', ugettext('Issuer name'), 20),
                ('issuer_email', ugettext('Issuer email'), 30),
                ('issuer_phone', ugettext('Issuer phone'), 30),
            ],
            ugettext('Customer'): [
                ('customer_name', ugettext('Customer name'), 20),
                ('customer_street', ugettext('Customer street'), 20),
                ('customer_zip', ugettext('Customer zip'), 15),
                ('customer_city', ugettext('Customer city'), 20),
                ('get_customer_country_display', ugettext('Customer country'), 20, None, lambda value: value()),
                ('customer_registration_id', ugettext('Customer reg. ID'), 20),
                ('customer_tax_id', ugettext('Customer tax ID'), 20),
                ('customer_vat_id', ugettext('Customer VAT ID'), 20),
                ('customer_additional_info', ugettext('Customer additional info'), 25, None, lambda ord_dict: ', '.join(': '.join([str(label), str(value)]) for label, value in ord_dict.items()) if ord_dict and not isinstance(ord_dict, str) else ''),
                ('customer_email', ugettext('Customer email'), 30),
                ('customer_phone', ugettext('Customer phone'), 30),
            ],
            # ugettext('Shipping'): [
            #     ('shipping_name', ugettext('Shipping name'), 20),
            #     ('shipping_street', ugettext('Shipping street'), 20),
            #     ('shipping_zip', ugettext('Shipping zip'), 15),
            #     ('shipping_city', ugettext('Shipping city'), 20),
            #     ('get_shipping_country_display', ugettext('Shipping country'), 20, None, lambda value: value()),
            #     ('get_delivery_method_display', ugettext('Delivery method'), 20, None, lambda value: value()),
            # ],
        })

    def get_whole_queryset(self, params):
        return super().get_whole_queryset(params) \
            .order_by('-created').distinct()
            # .prefetch_related(Prefetch('item_set', queryset=Item.objects.all())) \

    def get_worksheet_title(self, index=0):
        return ugettext('Invoices')

    def get_queryset(self):
        return self.queryset



# Generated by Django 2.2.9 on 2020-03-25 11:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0022_invoice_date_paid'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='related_invoices',
            field=models.ManyToManyField(related_name='_invoice_related_invoices_+', to='invoicing.Invoice'),
        ),
    ]



# Generated by Django 2.0.6 on 2018-10-01 08:25

from django.db import migrations
from internationalflavor.vat_number import VATNumberField


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0019_recalculate'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invoice',
            name='customer_vat_id',
            field=VATNumberField(blank=True, max_length=14),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='supplier_vat_id',
            field=VATNumberField(blank=True, max_length=14),
        ),
    ]



from django.conf.urls import url

from invoicing.views import InvoiceDetailView


app_name = 'invoicing'

urlpatterns = [
    url(r'^invoice/detail/(?P<pk>[-\d]+)/$', InvoiceDetailView.as_view(), name='invoice_detail'),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='invoice',
            name='discount',
        ),
        migrations.AddField(
            model_name='item',
            name='discount',
            field=models.DecimalField(default=0, verbose_name='discount (%)', max_digits=3, decimal_places=1),
            preserve_default=True,
        )
    ]



# Generated by Django 2.0.3 on 2018-05-31 16:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0008_recalculate'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invoice',
            name='vat',
            field=models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=10, null=True),
        ),
    ]



# Generated by Django 2.0.6 on 2018-06-19 12:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0017_auto_20180619_1349'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='attachments',
            field=models.TextField(blank=True, default=None, null=True),
        ),
    ]



from decimal import Decimal
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.core.validators import EMPTY_VALUES
from internationalflavor.vat_number import VATNumberValidator

from invoicing.taxation import TaxationPolicy


class EUTaxationPolicy(TaxationPolicy):
    """
    This taxation policy should be correct for all EU countries. It uses following rules:
        * if supplier country is not in EU - assert error,
        * return 'default tax' in cases:
            * if supplier country and customer country are the same,
            * if supplier country and customer country are not the same, but customer is private person from EU,
            * if supplier country and customer country are not the same, customer is company from EU, but his VAT ID is not valid according VIES system.
        * return tax not applicable (None) in cases:
            * if supplier country and customer country are not the same, customer is company from EU and his tax id is valid according VIES system.
            * if supplier country and customer country are not the same and customer is private person not from EU,
            * if supplier country and customer country are not the same and customer is company not from EU.


    Please note, that term "private person" refers in system to user that did not provide tax ID and
    ``company`` refers to user that provides it.

    """
    EU_COUNTRIES_RATES = {
        'AT': Decimal(20),  # Austria
        'BE': Decimal(21),  # Belgium
        'BG': Decimal(20),  # Bulgaria
        'CY': Decimal(19),  # Cyprus
        'CZ': Decimal(21),  # Czech Republic
        'DK': Decimal(25),  # Denmark
        'EE': Decimal(20),  # Estonia
        'FI': Decimal(24),  # Finland
        'FR': Decimal(20),  # France
        'DE': Decimal(19),  # Germany
        'GR': Decimal(24),  # Greece
        'HR': Decimal(25),  # Croatia
        'HU': Decimal(27),  # Hungary
        'IE': Decimal(21),  # Ireland
        'IT': Decimal(22),  # Italy
        'LV': Decimal(21),  # Latvia
        'LT': Decimal(21),  # Lithuania
        'LU': Decimal(17),  # Luxembourg
        'MT': Decimal(18),  # Malta
        'NL': Decimal(21),  # Netherlands
        'PL': Decimal(23),  # Poland
        'PT': Decimal(23),  # Portugal
        'RO': Decimal(19),  # Romania
        'SK': Decimal(20),  # Slovakia
        'SI': Decimal(22),  # Slovenia
        'ES': Decimal(21),  # Spain
        'SE': Decimal(25),  # Sweden
        'GB': Decimal(20),  # United Kingdom (Great Britain)
    }

    @classmethod
    def is_in_EU(cls, country_code):
        if country_code == 'GB':
            return False

        return country_code.upper() in cls.EU_COUNTRIES_RATES.keys()

    @classmethod
    def get_default_tax(cls, country_code=None):
        """
        Gets default tax rate.``

        :return: Decimal()
        """

        default_tax_rate = super().get_default_tax(country_code)

        # tax rate by country
        if country_code and not hasattr(settings, 'INVOICING_TAX_RATE'):
            return cls.EU_COUNTRIES_RATES.get(country_code, default_tax_rate)

        return default_tax_rate

    @classmethod
    def get_tax_rate(cls, vat_id, customer_country, supplier_country=None):
        if not supplier_country:
            supplier_country = cls.get_supplier_country_code()

        if not cls.is_in_EU(supplier_country):
            raise ImproperlyConfigured("EUTaxationPolicy requires that supplier country is in EU")

        if vat_id in EMPTY_VALUES:
            # We don't know VAT ID

            if customer_country in EMPTY_VALUES:
                # We don't know VAT ID or country
                return cls.get_default_tax(supplier_country)

            # Customer is not a company, we know his country

            if cls.is_in_EU(customer_country):
                # Customer (private person) is from a EU
                # He must pay full VAT of our country
                return cls.get_default_tax(supplier_country)
            else:
                # Customer (private person) is not from EU
                # charge back
                return None

        # Customer is company, we know country and VAT ID

        if customer_country.upper() == supplier_country.upper():
            # Company is from the same country as supplier
            # Normal tax
            return cls.get_default_tax(supplier_country)

        if not cls.is_in_EU(customer_country):
            # Company is not from EU
            # Charge back
            return None

        # Company is from other EU country
        use_vies_validator = getattr(settings, 'INVOICING_USE_VIES_VALIDATOR', True)

        if not use_vies_validator:
            # trust VAT ID is correct
            # Charge back
            return None

        try:
            # verify VAT ID in VIES
            VATNumberValidator(eu_only=True, vies_check=True)(vat_id)

            # Company is registered in VIES
            # Charge back
            return None
        except ValidationError:
            return cls.get_default_tax(supplier_country)



# Generated by Django 2.0.3 on 2018-03-27 14:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0011_auto_20180601_1459'),
    ]

    operations = [
        migrations.RenameField(
            model_name='invoice',
            old_name='number',
            new_name='sequence',
        ),
    ]



# Generated by Django 2.0.3 on 2018-05-29 14:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0006_auto_20180529_1224'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='total',
            field=models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=10),
        ),
        migrations.AddField(
            model_name='invoice',
            name='vat',
            field=models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=10),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations

try:
    # Django 3.1
    from django.db.models import JSONField
except ImportError:
    # older Django
    from django.contrib.postgres.fields import JSONField

import django.core.validators
import django_countries.fields
import django_iban.fields
import model_utils.fields
from internationalflavor.vat_number import VATNumberField


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Invoice',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('type', models.CharField(default=b'INVOICE', max_length=64, verbose_name='type', choices=[(b'INVOICE', 'Invoice'), (b'ADVANCE', 'Advance invoice'), (b'PROFORMA', 'Proforma invoice'), (b'VAT_CREDIT_NOTE', 'VAT credit note')])),
                ('number', models.IntegerField(db_index=True, verbose_name='number', blank=True)),
                ('full_number', models.CharField(max_length=128, blank=True)),
                ('status', models.CharField(default=b'NEW', max_length=64, verbose_name='status', choices=[(b'NEW', 'new'), (b'SENT', 'sent'), (b'RETURNED', 'returned'), (b'CANCELED', 'canceled'), (b'PAID', 'paid')])),
                ('subtitle', models.CharField(default=None, max_length=255, null=True, verbose_name='subtitle', blank=True)),
                ('language', models.CharField(max_length=10, verbose_name='language', choices=[(b'en', b'English')])),
                ('note', models.CharField(default='Thank you for using our services.', max_length=255, null=True, verbose_name='note', blank=True)),
                ('date_issue', models.DateField(verbose_name='issue date')),
                ('date_tax_point', models.DateField(help_text='time of supply', verbose_name='tax point date')),
                ('date_due', models.DateField(help_text='payment till', verbose_name='due date')),
                ('date_sent', model_utils.fields.MonitorField(default=None, null=True, when=set([b'SENT']), monitor=b'status', blank=True)),
                ('currency', models.CharField(max_length=10, verbose_name='currency', choices=[('AFN', 'Afghani'), ('DZD', 'Algerian Dinar'), ('ARS', 'Argentine Peso'), ('AMD', 'Armenian Dram'), ('AWG', 'Aruban Guilder'), ('AUD', 'Australian Dollar'), ('AZN', 'Azerbaijanian Manat'), ('BSD', 'Bahamian Dollar'), ('BHD', 'Bahraini Dinar'), ('THB', 'Baht'), ('BBD', 'Barbados Dollar'), ('BYR', 'Belarussian Ruble'), ('BZD', 'Belize Dollar'), ('BMD', 'Bermudian Dollar (customarily known as Bermuda Dollar)'), ('BTN', 'Bhutanese ngultrum'), ('VEF', 'Bolivar Fuerte'), ('XBA', 'Bond Markets Units European Composite Unit (EURCO)'), ('BRL', 'Brazilian Real'), ('BND', 'Brunei Dollar'), ('BGN', 'Bulgarian Lev'), ('BIF', 'Burundi Franc'), ('XOF', 'CFA Franc BCEAO'), ('XAF', 'CFA franc BEAC'), ('XPF', 'CFP Franc'), ('CAD', 'Canadian Dollar'), ('CVE', 'Cape Verde Escudo'), ('KYD', 'Cayman Islands Dollar'), ('CLP', 'Chilean peso'), ('XTS', 'Codes specifically reserved for testing purposes'), ('COP', 'Colombian peso'), ('KMF', 'Comoro Franc'), ('CDF', 'Congolese franc'), ('BAM', 'Convertible Marks'), ('NIO', 'Cordoba Oro'), ('CRC', 'Costa Rican Colon'), ('HRK', 'Croatian Kuna'), ('CUP', 'Cuban Peso'), ('CUC', 'Cuban convertible peso'), ('CZK', 'Czech Koruna'), ('GMD', 'Dalasi'), ('DKK', 'Danish Krone'), ('MKD', 'Denar'), ('DJF', 'Djibouti Franc'), ('STD', 'Dobra'), ('DOP', 'Dominican Peso'), ('VND', 'Dong'), ('XCD', 'East Caribbean Dollar'), ('EGP', 'Egyptian Pound'), ('ETB', 'Ethiopian Birr'), ('EUR', 'Euro'), ('XBB', 'European Monetary Unit (E.M.U.-6)'), ('XBD', 'European Unit of Account 17(E.U.A.-17)'), ('XBC', 'European Unit of Account 9(E.U.A.-9)'), ('FKP', 'Falkland Islands Pound'), ('FJD', 'Fiji Dollar'), ('HUF', 'Forint'), ('GHS', 'Ghana Cedi'), ('GIP', 'Gibraltar Pound'), ('XAU', 'Gold'), ('XFO', 'Gold-Franc'), ('PYG', 'Guarani'), ('GNF', 'Guinea Franc'), ('GYD', 'Guyana Dollar'), ('HTG', 'Haitian gourde'), ('HKD', 'Hong Kong Dollar'), ('UAH', 'Hryvnia'), ('ISK', 'Iceland Krona'), ('INR', 'Indian Rupee'), ('IRR', 'Iranian Rial'), ('IQD', 'Iraqi Dinar'), ('IMP', 'Isle of Man pount'), ('JMD', 'Jamaican Dollar'), ('JOD', 'Jordanian Dinar'), ('KES', 'Kenyan Shilling'), ('PGK', 'Kina'), ('LAK', 'Kip'), ('EEK', 'Kroon'), ('KWD', 'Kuwaiti Dinar'), ('MWK', 'Kwacha'), ('ZMK', 'Kwacha'), ('AOA', 'Kwanza'), ('MMK', 'Kyat'), ('GEL', 'Lari'), ('LVL', 'Latvian Lats'), ('LBP', 'Lebanese Pound'), ('ALL', 'Lek'), ('HNL', 'Lempira'), ('SLL', 'Leone'), ('LSL', 'Lesotho loti'), ('LRD', 'Liberian Dollar'), ('LYD', 'Libyan Dinar'), ('SZL', 'Lilangeni'), ('LTL', 'Lithuanian Litas'), ('MGA', 'Malagasy Ariary'), ('MYR', 'Malaysian Ringgit'), ('TMM', 'Manat'), ('MUR', 'Mauritius Rupee'), ('MZN', 'Metical'), ('MXN', 'Mexixan peso'), ('MDL', 'Moldovan Leu'), ('MAD', 'Moroccan Dirham'), ('NGN', 'Naira'), ('ERN', 'Nakfa'), ('NAD', 'Namibian Dollar'), ('NPR', 'Nepalese Rupee'), ('ANG', 'Netherlands Antillian Guilder'), ('ILS', 'New Israeli Sheqel'), ('RON', 'New Leu'), ('TWD', 'New Taiwan Dollar'), ('NZD', 'New Zealand Dollar'), ('KPW', 'North Korean Won'), ('NOK', 'Norwegian Krone'), ('PEN', 'Nuevo Sol'), ('MRO', 'Ouguiya'), ('TOP', 'Paanga'), ('PKR', 'Pakistan Rupee'), ('XPD', 'Palladium'), ('MOP', 'Pataca'), ('PHP', 'Philippine Peso'), ('XPT', 'Platinum'), ('GBP', 'Pound Sterling'), ('BWP', 'Pula'), ('QAR', 'Qatari Rial'), ('GTQ', 'Quetzal'), ('ZAR', 'Rand'), ('OMR', 'Rial Omani'), ('KHR', 'Riel'), ('MVR', 'Rufiyaa'), ('IDR', 'Rupiah'), ('RUB', 'Russian Ruble'), ('RWF', 'Rwanda Franc'), ('XDR', 'SDR'), ('SHP', 'Saint Helena Pound'), ('SAR', 'Saudi Riyal'), ('RSD', 'Serbian Dinar'), ('SCR', 'Seychelles Rupee'), ('XAG', 'Silver'), ('SGD', 'Singapore Dollar'), ('SKK', 'Slovak Koruna'), ('SBD', 'Solomon Islands Dollar'), ('KGS', 'Som'), ('SOS', 'Somali Shilling'), ('TJS', 'Somoni'), ('LKR', 'Sri Lanka Rupee'), ('SDG', 'Sudanese Pound'), ('SRD', 'Surinam Dollar'), ('SEK', 'Swedish Krona'), ('CHF', 'Swiss Franc'), ('SYP', 'Syrian Pound'), ('BDT', 'Taka'), ('WST', 'Tala'), ('TZS', 'Tanzanian Shilling'), ('KZT', 'Tenge'), ('TTD', 'Trinidad and Tobago Dollar'), ('MNT', 'Tugrik'), ('TND', 'Tunisian Dinar'), ('TRY', 'Turkish Lira'), ('TVD', 'Tuvalu dollar'), ('AED', 'UAE Dirham'), ('XFU', 'UIC-Franc'), ('USD', 'US Dollar'), ('UGX', 'Uganda Shilling'), ('UYU', 'Uruguayan peso'), ('UZS', 'Uzbekistan Sum'), ('VUV', 'Vatu'), ('KRW', 'Won'), ('YER', 'Yemeni Rial'), ('JPY', 'Yen'), ('CNY', 'Yuan Renminbi'), ('ZWD', 'Zimbabwe Dollar A/06'), ('ZWN', 'Zimbabwe dollar A/08'), ('ZWL', 'Zimbabwe dollar A/09'), ('PLN', 'Zloty')])),
                ('discount', models.DecimalField(default=0, verbose_name='discount (%)', max_digits=3, decimal_places=1)),
                ('credit', models.DecimalField(default=0, verbose_name='credit', max_digits=10, decimal_places=2)),
                ('payment_method', models.CharField(max_length=64, verbose_name='payment method', choices=[(b'BANK_TRANSFER', 'bank transfer'), (b'CASH', 'cash'), (b'CASH_ON_DELIVERY', 'cash on delivery'), (b'PAYMENT_CARD', 'payment card')])),
                ('constant_symbol', models.CharField(default=None, choices=[(b'0001', '0001 - Payments for goods based on legal and executable decision from legal authority'), (b'0008', '0008 - Cashless payments for goods'), (b'0038', '0038 - Cashless funds for wages'), (b'0058', '0058 - Cashless penalty and delinquency charges'), (b'0068', '0068 - Transfer of funds for wages and other personal costs'), (b'0138', '0138 - Cashless deductions at source'), (b'0168', '0168 - Cashless payments in loans'), (b'0178', '0178 - Sales from provided services'), (b'0298', '0298 - Other cashless transfers'), (b'0304', '0304 - Prior payments for services'), (b'0308', '0308 - Cashless payments for services'), (b'0358', '0358 - Payments dedicated to payout through post offices'), (b'0379', '0379 - Other income, income from postal order'), (b'0498', '0498 - Payments in loans'), (b'0558', '0558 - Cashless other financial payments'), (b'0934', '0934 - Benefits - prior payments'), (b'0968', '0968 - Other cashless transfers'), (b'1144', '1144 - Prior payment - advance'), (b'1148', '1148 - Payment - current advance'), (b'1744', '1744 - Accounting of tax at income tax of physical body and corporate body'), (b'1748', '1748 - Income tax of physical body and corporate body based on declared tax year'), (b'3118', '3118 - Insurance and empl. contrib. to insur. co. and the Labor Office'), (b'3344', '3344 - Penalty from message - prior'), (b'3348', '3348 - Penalty from message'), (b'3354', '3354 - Insurance payments by insurance companies'), (b'3558', '3558 - Cashless insurance payments by insurance companies'), (b'8147', '8147 - Payment (posted together with the instruction)')], max_length=64, blank=True, null=True, verbose_name='constant symbol')),
                ('variable_symbol', models.PositiveIntegerField(default=None, null=True, verbose_name='variable symbol', blank=True, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(9999999999)])),
                ('specific_symbol', models.PositiveIntegerField(default=None, null=True, verbose_name='specific symbol', blank=True, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(9999999999)])),
                ('reference', models.CharField(default=None, max_length=140, null=True, verbose_name='reference', blank=True)),
                ('bank_name', models.CharField(default=None, max_length=255, null=True, verbose_name='bank name', blank=True)),
                ('bank_street', models.CharField(default=None, max_length=255, null=True, verbose_name='bank street and number', blank=True)),
                ('bank_zip', models.CharField(default=None, max_length=255, null=True, verbose_name='bank ZIP', blank=True)),
                ('bank_city', models.CharField(default=None, max_length=255, null=True, verbose_name='bank city', blank=True)),
                ('bank_country', django_countries.fields.CountryField(default=None, choices=[('AF', 'Afghanistan'), ('AL', 'Albania'), ('DZ', 'Algeria'), ('AS', 'American Samoa'), ('AD', 'Andorra'), ('AO', 'Angola'), ('AI', 'Anguilla'), ('AQ', 'Antarctica'), ('AG', 'Antigua and Barbuda'), ('AR', 'Argentina'), ('AM', 'Armenia'), ('AW', 'Aruba'), ('AU', 'Australia'), ('AT', 'Austria'), ('AZ', 'Azerbaijan'), ('BS', 'Bahamas'), ('BH', 'Bahrain'), ('BD', 'Bangladesh'), ('BB', 'Barbados'), ('BY', 'Belarus'), ('BE', 'Belgium'), ('BZ', 'Belize'), ('BJ', 'Benin'), ('BM', 'Bermuda'), ('BT', 'Bhutan'), ('BO', 'Bolivia, Plurinational State of'), ('BQ', 'Bonaire, Sint Eustatius and Saba'), ('BA', 'Bosnia and Herzegovina'), ('BW', 'Botswana'), ('BV', 'Bouvet Island'), ('BR', 'Brazil'), ('IO', 'British Indian Ocean Territory'), ('BN', 'Brunei Darussalam'), ('BG', 'Bulgaria'), ('BF', 'Burkina Faso'), ('BI', 'Burundi'), ('KH', 'Cambodia'), ('CM', 'Cameroon'), ('CA', 'Canada'), ('CV', 'Cape Verde'), ('KY', 'Cayman Islands'), ('CF', 'Central African Republic'), ('TD', 'Chad'), ('CL', 'Chile'), ('CN', 'China'), ('CX', 'Christmas Island'), ('CC', 'Cocos (Keeling) Islands'), ('CO', 'Colombia'), ('KM', 'Comoros'), ('CG', 'Congo'), ('CD', 'Congo (the Democratic Republic of the)'), ('CK', 'Cook Islands'), ('CR', 'Costa Rica'), ('HR', 'Croatia'), ('CU', 'Cuba'), ('CW', 'Cura\xe7ao'), ('CY', 'Cyprus'), ('CZ', 'Czech Republic'), ('CI', "C\xf4te d'Ivoire"), ('DK', 'Denmark'), ('DJ', 'Djibouti'), ('DM', 'Dominica'), ('DO', 'Dominican Republic'), ('EC', 'Ecuador'), ('EG', 'Egypt'), ('SV', 'El Salvador'), ('GQ', 'Equatorial Guinea'), ('ER', 'Eritrea'), ('EE', 'Estonia'), ('ET', 'Ethiopia'), ('FK', 'Falkland Islands  [Malvinas]'), ('FO', 'Faroe Islands'), ('FJ', 'Fiji'), ('FI', 'Finland'), ('FR', 'France'), ('GF', 'French Guiana'), ('PF', 'French Polynesia'), ('TF', 'French Southern Territories'), ('GA', 'Gabon'), ('GM', 'Gambia (The)'), ('GE', 'Georgia'), ('DE', 'Germany'), ('GH', 'Ghana'), ('GI', 'Gibraltar'), ('GR', 'Greece'), ('GL', 'Greenland'), ('GD', 'Grenada'), ('GP', 'Guadeloupe'), ('GU', 'Guam'), ('GT', 'Guatemala'), ('GG', 'Guernsey'), ('GN', 'Guinea'), ('GW', 'Guinea-Bissau'), ('GY', 'Guyana'), ('HT', 'Haiti'), ('HM', 'Heard Island and McDonald Islands'), ('VA', 'Holy See  [Vatican City State]'), ('HN', 'Honduras'), ('HK', 'Hong Kong'), ('HU', 'Hungary'), ('IS', 'Iceland'), ('IN', 'India'), ('ID', 'Indonesia'), ('IR', 'Iran (the Islamic Republic of)'), ('IQ', 'Iraq'), ('IE', 'Ireland'), ('IM', 'Isle of Man'), ('IL', 'Israel'), ('IT', 'Italy'), ('JM', 'Jamaica'), ('JP', 'Japan'), ('JE', 'Jersey'), ('JO', 'Jordan'), ('KZ', 'Kazakhstan'), ('KE', 'Kenya'), ('KI', 'Kiribati'), ('KP', "Korea (the Democratic People's Republic of)"), ('KR', 'Korea (the Republic of)'), ('KW', 'Kuwait'), ('KG', 'Kyrgyzstan'), ('LA', "Lao People's Democratic Republic"), ('LV', 'Latvia'), ('LB', 'Lebanon'), ('LS', 'Lesotho'), ('LR', 'Liberia'), ('LY', 'Libya'), ('LI', 'Liechtenstein'), ('LT', 'Lithuania'), ('LU', 'Luxembourg'), ('MO', 'Macao'), ('MK', 'Macedonia (the former Yugoslav Republic of)'), ('MG', 'Madagascar'), ('MW', 'Malawi'), ('MY', 'Malaysia'), ('MV', 'Maldives'), ('ML', 'Mali'), ('MT', 'Malta'), ('MH', 'Marshall Islands'), ('MQ', 'Martinique'), ('MR', 'Mauritania'), ('MU', 'Mauritius'), ('YT', 'Mayotte'), ('MX', 'Mexico'), ('FM', 'Micronesia (the Federated States of)'), ('MD', 'Moldova (the Republic of)'), ('MC', 'Monaco'), ('MN', 'Mongolia'), ('ME', 'Montenegro'), ('MS', 'Montserrat'), ('MA', 'Morocco'), ('MZ', 'Mozambique'), ('MM', 'Myanmar'), ('NA', 'Namibia'), ('NR', 'Nauru'), ('NP', 'Nepal'), ('NL', 'Netherlands'), ('NC', 'New Caledonia'), ('NZ', 'New Zealand'), ('NI', 'Nicaragua'), ('NE', 'Niger'), ('NG', 'Nigeria'), ('NU', 'Niue'), ('NF', 'Norfolk Island'), ('MP', 'Northern Mariana Islands'), ('NO', 'Norway'), ('OM', 'Oman'), ('PK', 'Pakistan'), ('PW', 'Palau'), ('PS', 'Palestine, State of'), ('PA', 'Panama'), ('PG', 'Papua New Guinea'), ('PY', 'Paraguay'), ('PE', 'Peru'), ('PH', 'Philippines'), ('PN', 'Pitcairn'), ('PL', 'Poland'), ('PT', 'Portugal'), ('PR', 'Puerto Rico'), ('QA', 'Qatar'), ('RO', 'Romania'), ('RU', 'Russian Federation'), ('RW', 'Rwanda'), ('RE', 'R\xe9union'), ('BL', 'Saint Barth\xe9lemy'), ('SH', 'Saint Helena, Ascension and Tristan da Cunha'), ('KN', 'Saint Kitts and Nevis'), ('LC', 'Saint Lucia'), ('MF', 'Saint Martin (French part)'), ('PM', 'Saint Pierre and Miquelon'), ('VC', 'Saint Vincent and the Grenadines'), ('WS', 'Samoa'), ('SM', 'San Marino'), ('ST', 'Sao Tome and Principe'), ('SA', 'Saudi Arabia'), ('SN', 'Senegal'), ('RS', 'Serbia'), ('SC', 'Seychelles'), ('SL', 'Sierra Leone'), ('SG', 'Singapore'), ('SX', 'Sint Maarten (Dutch part)'), ('SK', 'Slovakia'), ('SI', 'Slovenia'), ('SB', 'Solomon Islands'), ('SO', 'Somalia'), ('ZA', 'South Africa'), ('GS', 'South Georgia and the South Sandwich Islands'), ('SS', 'South Sudan'), ('ES', 'Spain'), ('LK', 'Sri Lanka'), ('SD', 'Sudan'), ('SR', 'Suriname'), ('SJ', 'Svalbard and Jan Mayen'), ('SZ', 'Swaziland'), ('SE', 'Sweden'), ('CH', 'Switzerland'), ('SY', 'Syrian Arab Republic'), ('TW', 'Taiwan (Province of China)'), ('TJ', 'Tajikistan'), ('TZ', 'Tanzania, United Republic of'), ('TH', 'Thailand'), ('TL', 'Timor-Leste'), ('TG', 'Togo'), ('TK', 'Tokelau'), ('TO', 'Tonga'), ('TT', 'Trinidad and Tobago'), ('TN', 'Tunisia'), ('TR', 'Turkey'), ('TM', 'Turkmenistan'), ('TC', 'Turks and Caicos Islands'), ('TV', 'Tuvalu'), ('UG', 'Uganda'), ('UA', 'Ukraine'), ('AE', 'United Arab Emirates'), ('GB', 'United Kingdom'), ('US', 'United States'), ('UM', 'United States Minor Outlying Islands'), ('UY', 'Uruguay'), ('UZ', 'Uzbekistan'), ('VU', 'Vanuatu'), ('VE', 'Venezuela, Bolivarian Republic of'), ('VN', 'Viet Nam'), ('VG', 'Virgin Islands (British)'), ('VI', 'Virgin Islands (U.S.)'), ('WF', 'Wallis and Futuna'), ('EH', 'Western Sahara'), ('YE', 'Yemen'), ('ZM', 'Zambia'), ('ZW', 'Zimbabwe'), ('AX', '\xc5land Islands')], max_length=2, blank=True, null=True, verbose_name='bank country')),
                ('bank_iban', django_iban.fields.IBANField(default=None, max_length=34, verbose_name='Account number (IBAN)')),
                ('bank_swift_bic', django_iban.fields.SWIFTBICField(default=None, max_length=11, verbose_name='Bank SWIFT / BIC')),
                ('supplier_name', models.CharField(default=None, max_length=255, verbose_name='supplier name')),
                ('supplier_street', models.CharField(default=None, max_length=255, null=True, verbose_name='supplier street and number', blank=True)),
                ('supplier_zip', models.CharField(default=None, max_length=255, null=True, verbose_name='supplier ZIP', blank=True)),
                ('supplier_city', models.CharField(default=None, max_length=255, null=True, verbose_name='supplier city', blank=True)),
                ('supplier_country', django_countries.fields.CountryField(default=None, max_length=2, verbose_name='supplier country', choices=[('AF', 'Afghanistan'), ('AL', 'Albania'), ('DZ', 'Algeria'), ('AS', 'American Samoa'), ('AD', 'Andorra'), ('AO', 'Angola'), ('AI', 'Anguilla'), ('AQ', 'Antarctica'), ('AG', 'Antigua and Barbuda'), ('AR', 'Argentina'), ('AM', 'Armenia'), ('AW', 'Aruba'), ('AU', 'Australia'), ('AT', 'Austria'), ('AZ', 'Azerbaijan'), ('BS', 'Bahamas'), ('BH', 'Bahrain'), ('BD', 'Bangladesh'), ('BB', 'Barbados'), ('BY', 'Belarus'), ('BE', 'Belgium'), ('BZ', 'Belize'), ('BJ', 'Benin'), ('BM', 'Bermuda'), ('BT', 'Bhutan'), ('BO', 'Bolivia, Plurinational State of'), ('BQ', 'Bonaire, Sint Eustatius and Saba'), ('BA', 'Bosnia and Herzegovina'), ('BW', 'Botswana'), ('BV', 'Bouvet Island'), ('BR', 'Brazil'), ('IO', 'British Indian Ocean Territory'), ('BN', 'Brunei Darussalam'), ('BG', 'Bulgaria'), ('BF', 'Burkina Faso'), ('BI', 'Burundi'), ('KH', 'Cambodia'), ('CM', 'Cameroon'), ('CA', 'Canada'), ('CV', 'Cape Verde'), ('KY', 'Cayman Islands'), ('CF', 'Central African Republic'), ('TD', 'Chad'), ('CL', 'Chile'), ('CN', 'China'), ('CX', 'Christmas Island'), ('CC', 'Cocos (Keeling) Islands'), ('CO', 'Colombia'), ('KM', 'Comoros'), ('CG', 'Congo'), ('CD', 'Congo (the Democratic Republic of the)'), ('CK', 'Cook Islands'), ('CR', 'Costa Rica'), ('HR', 'Croatia'), ('CU', 'Cuba'), ('CW', 'Cura\xe7ao'), ('CY', 'Cyprus'), ('CZ', 'Czech Republic'), ('CI', "C\xf4te d'Ivoire"), ('DK', 'Denmark'), ('DJ', 'Djibouti'), ('DM', 'Dominica'), ('DO', 'Dominican Republic'), ('EC', 'Ecuador'), ('EG', 'Egypt'), ('SV', 'El Salvador'), ('GQ', 'Equatorial Guinea'), ('ER', 'Eritrea'), ('EE', 'Estonia'), ('ET', 'Ethiopia'), ('FK', 'Falkland Islands  [Malvinas]'), ('FO', 'Faroe Islands'), ('FJ', 'Fiji'), ('FI', 'Finland'), ('FR', 'France'), ('GF', 'French Guiana'), ('PF', 'French Polynesia'), ('TF', 'French Southern Territories'), ('GA', 'Gabon'), ('GM', 'Gambia (The)'), ('GE', 'Georgia'), ('DE', 'Germany'), ('GH', 'Ghana'), ('GI', 'Gibraltar'), ('GR', 'Greece'), ('GL', 'Greenland'), ('GD', 'Grenada'), ('GP', 'Guadeloupe'), ('GU', 'Guam'), ('GT', 'Guatemala'), ('GG', 'Guernsey'), ('GN', 'Guinea'), ('GW', 'Guinea-Bissau'), ('GY', 'Guyana'), ('HT', 'Haiti'), ('HM', 'Heard Island and McDonald Islands'), ('VA', 'Holy See  [Vatican City State]'), ('HN', 'Honduras'), ('HK', 'Hong Kong'), ('HU', 'Hungary'), ('IS', 'Iceland'), ('IN', 'India'), ('ID', 'Indonesia'), ('IR', 'Iran (the Islamic Republic of)'), ('IQ', 'Iraq'), ('IE', 'Ireland'), ('IM', 'Isle of Man'), ('IL', 'Israel'), ('IT', 'Italy'), ('JM', 'Jamaica'), ('JP', 'Japan'), ('JE', 'Jersey'), ('JO', 'Jordan'), ('KZ', 'Kazakhstan'), ('KE', 'Kenya'), ('KI', 'Kiribati'), ('KP', "Korea (the Democratic People's Republic of)"), ('KR', 'Korea (the Republic of)'), ('KW', 'Kuwait'), ('KG', 'Kyrgyzstan'), ('LA', "Lao People's Democratic Republic"), ('LV', 'Latvia'), ('LB', 'Lebanon'), ('LS', 'Lesotho'), ('LR', 'Liberia'), ('LY', 'Libya'), ('LI', 'Liechtenstein'), ('LT', 'Lithuania'), ('LU', 'Luxembourg'), ('MO', 'Macao'), ('MK', 'Macedonia (the former Yugoslav Republic of)'), ('MG', 'Madagascar'), ('MW', 'Malawi'), ('MY', 'Malaysia'), ('MV', 'Maldives'), ('ML', 'Mali'), ('MT', 'Malta'), ('MH', 'Marshall Islands'), ('MQ', 'Martinique'), ('MR', 'Mauritania'), ('MU', 'Mauritius'), ('YT', 'Mayotte'), ('MX', 'Mexico'), ('FM', 'Micronesia (the Federated States of)'), ('MD', 'Moldova (the Republic of)'), ('MC', 'Monaco'), ('MN', 'Mongolia'), ('ME', 'Montenegro'), ('MS', 'Montserrat'), ('MA', 'Morocco'), ('MZ', 'Mozambique'), ('MM', 'Myanmar'), ('NA', 'Namibia'), ('NR', 'Nauru'), ('NP', 'Nepal'), ('NL', 'Netherlands'), ('NC', 'New Caledonia'), ('NZ', 'New Zealand'), ('NI', 'Nicaragua'), ('NE', 'Niger'), ('NG', 'Nigeria'), ('NU', 'Niue'), ('NF', 'Norfolk Island'), ('MP', 'Northern Mariana Islands'), ('NO', 'Norway'), ('OM', 'Oman'), ('PK', 'Pakistan'), ('PW', 'Palau'), ('PS', 'Palestine, State of'), ('PA', 'Panama'), ('PG', 'Papua New Guinea'), ('PY', 'Paraguay'), ('PE', 'Peru'), ('PH', 'Philippines'), ('PN', 'Pitcairn'), ('PL', 'Poland'), ('PT', 'Portugal'), ('PR', 'Puerto Rico'), ('QA', 'Qatar'), ('RO', 'Romania'), ('RU', 'Russian Federation'), ('RW', 'Rwanda'), ('RE', 'R\xe9union'), ('BL', 'Saint Barth\xe9lemy'), ('SH', 'Saint Helena, Ascension and Tristan da Cunha'), ('KN', 'Saint Kitts and Nevis'), ('LC', 'Saint Lucia'), ('MF', 'Saint Martin (French part)'), ('PM', 'Saint Pierre and Miquelon'), ('VC', 'Saint Vincent and the Grenadines'), ('WS', 'Samoa'), ('SM', 'San Marino'), ('ST', 'Sao Tome and Principe'), ('SA', 'Saudi Arabia'), ('SN', 'Senegal'), ('RS', 'Serbia'), ('SC', 'Seychelles'), ('SL', 'Sierra Leone'), ('SG', 'Singapore'), ('SX', 'Sint Maarten (Dutch part)'), ('SK', 'Slovakia'), ('SI', 'Slovenia'), ('SB', 'Solomon Islands'), ('SO', 'Somalia'), ('ZA', 'South Africa'), ('GS', 'South Georgia and the South Sandwich Islands'), ('SS', 'South Sudan'), ('ES', 'Spain'), ('LK', 'Sri Lanka'), ('SD', 'Sudan'), ('SR', 'Suriname'), ('SJ', 'Svalbard and Jan Mayen'), ('SZ', 'Swaziland'), ('SE', 'Sweden'), ('CH', 'Switzerland'), ('SY', 'Syrian Arab Republic'), ('TW', 'Taiwan (Province of China)'), ('TJ', 'Tajikistan'), ('TZ', 'Tanzania, United Republic of'), ('TH', 'Thailand'), ('TL', 'Timor-Leste'), ('TG', 'Togo'), ('TK', 'Tokelau'), ('TO', 'Tonga'), ('TT', 'Trinidad and Tobago'), ('TN', 'Tunisia'), ('TR', 'Turkey'), ('TM', 'Turkmenistan'), ('TC', 'Turks and Caicos Islands'), ('TV', 'Tuvalu'), ('UG', 'Uganda'), ('UA', 'Ukraine'), ('AE', 'United Arab Emirates'), ('GB', 'United Kingdom'), ('US', 'United States'), ('UM', 'United States Minor Outlying Islands'), ('UY', 'Uruguay'), ('UZ', 'Uzbekistan'), ('VU', 'Vanuatu'), ('VE', 'Venezuela, Bolivarian Republic of'), ('VN', 'Viet Nam'), ('VG', 'Virgin Islands (British)'), ('VI', 'Virgin Islands (U.S.)'), ('WF', 'Wallis and Futuna'), ('EH', 'Western Sahara'), ('YE', 'Yemen'), ('ZM', 'Zambia'), ('ZW', 'Zimbabwe'), ('AX', '\xc5land Islands')])),
                ('supplier_registration_id', models.CharField(default=None, max_length=255, null=True, verbose_name='supplier Reg. No.', blank=True)),
                ('supplier_tax_id', models.CharField(default=None, max_length=255, null=True, verbose_name='supplier Tax No.', blank=True)),
                ('supplier_vat_id', VATNumberField(default=None, max_length=13, null=True, verbose_name='supplier VAT No.', blank=True)),
                ('supplier_additional_info', models.TextField(default=None, null=True, verbose_name='supplier additional information', blank=True)),
                ('issuer_name', models.CharField(default=None, max_length=255, null=True, verbose_name='issuer name', blank=True)),
                ('issuer_email', models.EmailField(default=None, max_length=254, null=True, verbose_name='issuer email', blank=True)),
                ('issuer_phone', models.CharField(default=None, max_length=255, null=True, verbose_name='issuer phone', blank=True)),
                ('customer_name', models.CharField(max_length=255, verbose_name='customer name')),
                ('customer_street', models.CharField(default=None, max_length=255, null=True, verbose_name='customer street and number', blank=True)),
                ('customer_zip', models.CharField(default=None, max_length=255, null=True, verbose_name='customer ZIP', blank=True)),
                ('customer_city', models.CharField(default=None, max_length=255, null=True, verbose_name='customer city', blank=True)),
                ('customer_country', django_countries.fields.CountryField(max_length=2, verbose_name='customer country', choices=[('AF', 'Afghanistan'), ('AL', 'Albania'), ('DZ', 'Algeria'), ('AS', 'American Samoa'), ('AD', 'Andorra'), ('AO', 'Angola'), ('AI', 'Anguilla'), ('AQ', 'Antarctica'), ('AG', 'Antigua and Barbuda'), ('AR', 'Argentina'), ('AM', 'Armenia'), ('AW', 'Aruba'), ('AU', 'Australia'), ('AT', 'Austria'), ('AZ', 'Azerbaijan'), ('BS', 'Bahamas'), ('BH', 'Bahrain'), ('BD', 'Bangladesh'), ('BB', 'Barbados'), ('BY', 'Belarus'), ('BE', 'Belgium'), ('BZ', 'Belize'), ('BJ', 'Benin'), ('BM', 'Bermuda'), ('BT', 'Bhutan'), ('BO', 'Bolivia, Plurinational State of'), ('BQ', 'Bonaire, Sint Eustatius and Saba'), ('BA', 'Bosnia and Herzegovina'), ('BW', 'Botswana'), ('BV', 'Bouvet Island'), ('BR', 'Brazil'), ('IO', 'British Indian Ocean Territory'), ('BN', 'Brunei Darussalam'), ('BG', 'Bulgaria'), ('BF', 'Burkina Faso'), ('BI', 'Burundi'), ('KH', 'Cambodia'), ('CM', 'Cameroon'), ('CA', 'Canada'), ('CV', 'Cape Verde'), ('KY', 'Cayman Islands'), ('CF', 'Central African Republic'), ('TD', 'Chad'), ('CL', 'Chile'), ('CN', 'China'), ('CX', 'Christmas Island'), ('CC', 'Cocos (Keeling) Islands'), ('CO', 'Colombia'), ('KM', 'Comoros'), ('CG', 'Congo'), ('CD', 'Congo (the Democratic Republic of the)'), ('CK', 'Cook Islands'), ('CR', 'Costa Rica'), ('HR', 'Croatia'), ('CU', 'Cuba'), ('CW', 'Cura\xe7ao'), ('CY', 'Cyprus'), ('CZ', 'Czech Republic'), ('CI', "C\xf4te d'Ivoire"), ('DK', 'Denmark'), ('DJ', 'Djibouti'), ('DM', 'Dominica'), ('DO', 'Dominican Republic'), ('EC', 'Ecuador'), ('EG', 'Egypt'), ('SV', 'El Salvador'), ('GQ', 'Equatorial Guinea'), ('ER', 'Eritrea'), ('EE', 'Estonia'), ('ET', 'Ethiopia'), ('FK', 'Falkland Islands  [Malvinas]'), ('FO', 'Faroe Islands'), ('FJ', 'Fiji'), ('FI', 'Finland'), ('FR', 'France'), ('GF', 'French Guiana'), ('PF', 'French Polynesia'), ('TF', 'French Southern Territories'), ('GA', 'Gabon'), ('GM', 'Gambia (The)'), ('GE', 'Georgia'), ('DE', 'Germany'), ('GH', 'Ghana'), ('GI', 'Gibraltar'), ('GR', 'Greece'), ('GL', 'Greenland'), ('GD', 'Grenada'), ('GP', 'Guadeloupe'), ('GU', 'Guam'), ('GT', 'Guatemala'), ('GG', 'Guernsey'), ('GN', 'Guinea'), ('GW', 'Guinea-Bissau'), ('GY', 'Guyana'), ('HT', 'Haiti'), ('HM', 'Heard Island and McDonald Islands'), ('VA', 'Holy See  [Vatican City State]'), ('HN', 'Honduras'), ('HK', 'Hong Kong'), ('HU', 'Hungary'), ('IS', 'Iceland'), ('IN', 'India'), ('ID', 'Indonesia'), ('IR', 'Iran (the Islamic Republic of)'), ('IQ', 'Iraq'), ('IE', 'Ireland'), ('IM', 'Isle of Man'), ('IL', 'Israel'), ('IT', 'Italy'), ('JM', 'Jamaica'), ('JP', 'Japan'), ('JE', 'Jersey'), ('JO', 'Jordan'), ('KZ', 'Kazakhstan'), ('KE', 'Kenya'), ('KI', 'Kiribati'), ('KP', "Korea (the Democratic People's Republic of)"), ('KR', 'Korea (the Republic of)'), ('KW', 'Kuwait'), ('KG', 'Kyrgyzstan'), ('LA', "Lao People's Democratic Republic"), ('LV', 'Latvia'), ('LB', 'Lebanon'), ('LS', 'Lesotho'), ('LR', 'Liberia'), ('LY', 'Libya'), ('LI', 'Liechtenstein'), ('LT', 'Lithuania'), ('LU', 'Luxembourg'), ('MO', 'Macao'), ('MK', 'Macedonia (the former Yugoslav Republic of)'), ('MG', 'Madagascar'), ('MW', 'Malawi'), ('MY', 'Malaysia'), ('MV', 'Maldives'), ('ML', 'Mali'), ('MT', 'Malta'), ('MH', 'Marshall Islands'), ('MQ', 'Martinique'), ('MR', 'Mauritania'), ('MU', 'Mauritius'), ('YT', 'Mayotte'), ('MX', 'Mexico'), ('FM', 'Micronesia (the Federated States of)'), ('MD', 'Moldova (the Republic of)'), ('MC', 'Monaco'), ('MN', 'Mongolia'), ('ME', 'Montenegro'), ('MS', 'Montserrat'), ('MA', 'Morocco'), ('MZ', 'Mozambique'), ('MM', 'Myanmar'), ('NA', 'Namibia'), ('NR', 'Nauru'), ('NP', 'Nepal'), ('NL', 'Netherlands'), ('NC', 'New Caledonia'), ('NZ', 'New Zealand'), ('NI', 'Nicaragua'), ('NE', 'Niger'), ('NG', 'Nigeria'), ('NU', 'Niue'), ('NF', 'Norfolk Island'), ('MP', 'Northern Mariana Islands'), ('NO', 'Norway'), ('OM', 'Oman'), ('PK', 'Pakistan'), ('PW', 'Palau'), ('PS', 'Palestine, State of'), ('PA', 'Panama'), ('PG', 'Papua New Guinea'), ('PY', 'Paraguay'), ('PE', 'Peru'), ('PH', 'Philippines'), ('PN', 'Pitcairn'), ('PL', 'Poland'), ('PT', 'Portugal'), ('PR', 'Puerto Rico'), ('QA', 'Qatar'), ('RO', 'Romania'), ('RU', 'Russian Federation'), ('RW', 'Rwanda'), ('RE', 'R\xe9union'), ('BL', 'Saint Barth\xe9lemy'), ('SH', 'Saint Helena, Ascension and Tristan da Cunha'), ('KN', 'Saint Kitts and Nevis'), ('LC', 'Saint Lucia'), ('MF', 'Saint Martin (French part)'), ('PM', 'Saint Pierre and Miquelon'), ('VC', 'Saint Vincent and the Grenadines'), ('WS', 'Samoa'), ('SM', 'San Marino'), ('ST', 'Sao Tome and Principe'), ('SA', 'Saudi Arabia'), ('SN', 'Senegal'), ('RS', 'Serbia'), ('SC', 'Seychelles'), ('SL', 'Sierra Leone'), ('SG', 'Singapore'), ('SX', 'Sint Maarten (Dutch part)'), ('SK', 'Slovakia'), ('SI', 'Slovenia'), ('SB', 'Solomon Islands'), ('SO', 'Somalia'), ('ZA', 'South Africa'), ('GS', 'South Georgia and the South Sandwich Islands'), ('SS', 'South Sudan'), ('ES', 'Spain'), ('LK', 'Sri Lanka'), ('SD', 'Sudan'), ('SR', 'Suriname'), ('SJ', 'Svalbard and Jan Mayen'), ('SZ', 'Swaziland'), ('SE', 'Sweden'), ('CH', 'Switzerland'), ('SY', 'Syrian Arab Republic'), ('TW', 'Taiwan (Province of China)'), ('TJ', 'Tajikistan'), ('TZ', 'Tanzania, United Republic of'), ('TH', 'Thailand'), ('TL', 'Timor-Leste'), ('TG', 'Togo'), ('TK', 'Tokelau'), ('TO', 'Tonga'), ('TT', 'Trinidad and Tobago'), ('TN', 'Tunisia'), ('TR', 'Turkey'), ('TM', 'Turkmenistan'), ('TC', 'Turks and Caicos Islands'), ('TV', 'Tuvalu'), ('UG', 'Uganda'), ('UA', 'Ukraine'), ('AE', 'United Arab Emirates'), ('GB', 'United Kingdom'), ('US', 'United States'), ('UM', 'United States Minor Outlying Islands'), ('UY', 'Uruguay'), ('UZ', 'Uzbekistan'), ('VU', 'Vanuatu'), ('VE', 'Venezuela, Bolivarian Republic of'), ('VN', 'Viet Nam'), ('VG', 'Virgin Islands (British)'), ('VI', 'Virgin Islands (U.S.)'), ('WF', 'Wallis and Futuna'), ('EH', 'Western Sahara'), ('YE', 'Yemen'), ('ZM', 'Zambia'), ('ZW', 'Zimbabwe'), ('AX', '\xc5land Islands')])),
                ('customer_registration_id', models.CharField(default=None, max_length=255, null=True, verbose_name='customer Reg. No.', blank=True)),
                ('customer_tax_id', models.CharField(default=None, max_length=255, null=True, verbose_name='customer Tax No.', blank=True)),
                ('customer_vat_id', VATNumberField(default=None, max_length=13, null=True, verbose_name='customer VAT No.', blank=True)),
                ('customer_additional_info', JSONField(default=None, null=True, verbose_name='customer additional information', blank=True)),
                ('shipping_name', models.CharField(default=None, max_length=255, null=True, verbose_name='shipping name', blank=True)),
                ('shipping_street', models.CharField(default=None, max_length=255, null=True, verbose_name='shipping street and number', blank=True)),
                ('shipping_zip', models.CharField(default=None, max_length=255, null=True, verbose_name='shipping ZIP', blank=True)),
                ('shipping_city', models.CharField(default=None, max_length=255, null=True, verbose_name='shipping city', blank=True)),
                ('shipping_country', django_countries.fields.CountryField(default=None, choices=[('AF', 'Afghanistan'), ('AL', 'Albania'), ('DZ', 'Algeria'), ('AS', 'American Samoa'), ('AD', 'Andorra'), ('AO', 'Angola'), ('AI', 'Anguilla'), ('AQ', 'Antarctica'), ('AG', 'Antigua and Barbuda'), ('AR', 'Argentina'), ('AM', 'Armenia'), ('AW', 'Aruba'), ('AU', 'Australia'), ('AT', 'Austria'), ('AZ', 'Azerbaijan'), ('BS', 'Bahamas'), ('BH', 'Bahrain'), ('BD', 'Bangladesh'), ('BB', 'Barbados'), ('BY', 'Belarus'), ('BE', 'Belgium'), ('BZ', 'Belize'), ('BJ', 'Benin'), ('BM', 'Bermuda'), ('BT', 'Bhutan'), ('BO', 'Bolivia, Plurinational State of'), ('BQ', 'Bonaire, Sint Eustatius and Saba'), ('BA', 'Bosnia and Herzegovina'), ('BW', 'Botswana'), ('BV', 'Bouvet Island'), ('BR', 'Brazil'), ('IO', 'British Indian Ocean Territory'), ('BN', 'Brunei Darussalam'), ('BG', 'Bulgaria'), ('BF', 'Burkina Faso'), ('BI', 'Burundi'), ('KH', 'Cambodia'), ('CM', 'Cameroon'), ('CA', 'Canada'), ('CV', 'Cape Verde'), ('KY', 'Cayman Islands'), ('CF', 'Central African Republic'), ('TD', 'Chad'), ('CL', 'Chile'), ('CN', 'China'), ('CX', 'Christmas Island'), ('CC', 'Cocos (Keeling) Islands'), ('CO', 'Colombia'), ('KM', 'Comoros'), ('CG', 'Congo'), ('CD', 'Congo (the Democratic Republic of the)'), ('CK', 'Cook Islands'), ('CR', 'Costa Rica'), ('HR', 'Croatia'), ('CU', 'Cuba'), ('CW', 'Cura\xe7ao'), ('CY', 'Cyprus'), ('CZ', 'Czech Republic'), ('CI', "C\xf4te d'Ivoire"), ('DK', 'Denmark'), ('DJ', 'Djibouti'), ('DM', 'Dominica'), ('DO', 'Dominican Republic'), ('EC', 'Ecuador'), ('EG', 'Egypt'), ('SV', 'El Salvador'), ('GQ', 'Equatorial Guinea'), ('ER', 'Eritrea'), ('EE', 'Estonia'), ('ET', 'Ethiopia'), ('FK', 'Falkland Islands  [Malvinas]'), ('FO', 'Faroe Islands'), ('FJ', 'Fiji'), ('FI', 'Finland'), ('FR', 'France'), ('GF', 'French Guiana'), ('PF', 'French Polynesia'), ('TF', 'French Southern Territories'), ('GA', 'Gabon'), ('GM', 'Gambia (The)'), ('GE', 'Georgia'), ('DE', 'Germany'), ('GH', 'Ghana'), ('GI', 'Gibraltar'), ('GR', 'Greece'), ('GL', 'Greenland'), ('GD', 'Grenada'), ('GP', 'Guadeloupe'), ('GU', 'Guam'), ('GT', 'Guatemala'), ('GG', 'Guernsey'), ('GN', 'Guinea'), ('GW', 'Guinea-Bissau'), ('GY', 'Guyana'), ('HT', 'Haiti'), ('HM', 'Heard Island and McDonald Islands'), ('VA', 'Holy See  [Vatican City State]'), ('HN', 'Honduras'), ('HK', 'Hong Kong'), ('HU', 'Hungary'), ('IS', 'Iceland'), ('IN', 'India'), ('ID', 'Indonesia'), ('IR', 'Iran (the Islamic Republic of)'), ('IQ', 'Iraq'), ('IE', 'Ireland'), ('IM', 'Isle of Man'), ('IL', 'Israel'), ('IT', 'Italy'), ('JM', 'Jamaica'), ('JP', 'Japan'), ('JE', 'Jersey'), ('JO', 'Jordan'), ('KZ', 'Kazakhstan'), ('KE', 'Kenya'), ('KI', 'Kiribati'), ('KP', "Korea (the Democratic People's Republic of)"), ('KR', 'Korea (the Republic of)'), ('KW', 'Kuwait'), ('KG', 'Kyrgyzstan'), ('LA', "Lao People's Democratic Republic"), ('LV', 'Latvia'), ('LB', 'Lebanon'), ('LS', 'Lesotho'), ('LR', 'Liberia'), ('LY', 'Libya'), ('LI', 'Liechtenstein'), ('LT', 'Lithuania'), ('LU', 'Luxembourg'), ('MO', 'Macao'), ('MK', 'Macedonia (the former Yugoslav Republic of)'), ('MG', 'Madagascar'), ('MW', 'Malawi'), ('MY', 'Malaysia'), ('MV', 'Maldives'), ('ML', 'Mali'), ('MT', 'Malta'), ('MH', 'Marshall Islands'), ('MQ', 'Martinique'), ('MR', 'Mauritania'), ('MU', 'Mauritius'), ('YT', 'Mayotte'), ('MX', 'Mexico'), ('FM', 'Micronesia (the Federated States of)'), ('MD', 'Moldova (the Republic of)'), ('MC', 'Monaco'), ('MN', 'Mongolia'), ('ME', 'Montenegro'), ('MS', 'Montserrat'), ('MA', 'Morocco'), ('MZ', 'Mozambique'), ('MM', 'Myanmar'), ('NA', 'Namibia'), ('NR', 'Nauru'), ('NP', 'Nepal'), ('NL', 'Netherlands'), ('NC', 'New Caledonia'), ('NZ', 'New Zealand'), ('NI', 'Nicaragua'), ('NE', 'Niger'), ('NG', 'Nigeria'), ('NU', 'Niue'), ('NF', 'Norfolk Island'), ('MP', 'Northern Mariana Islands'), ('NO', 'Norway'), ('OM', 'Oman'), ('PK', 'Pakistan'), ('PW', 'Palau'), ('PS', 'Palestine, State of'), ('PA', 'Panama'), ('PG', 'Papua New Guinea'), ('PY', 'Paraguay'), ('PE', 'Peru'), ('PH', 'Philippines'), ('PN', 'Pitcairn'), ('PL', 'Poland'), ('PT', 'Portugal'), ('PR', 'Puerto Rico'), ('QA', 'Qatar'), ('RO', 'Romania'), ('RU', 'Russian Federation'), ('RW', 'Rwanda'), ('RE', 'R\xe9union'), ('BL', 'Saint Barth\xe9lemy'), ('SH', 'Saint Helena, Ascension and Tristan da Cunha'), ('KN', 'Saint Kitts and Nevis'), ('LC', 'Saint Lucia'), ('MF', 'Saint Martin (French part)'), ('PM', 'Saint Pierre and Miquelon'), ('VC', 'Saint Vincent and the Grenadines'), ('WS', 'Samoa'), ('SM', 'San Marino'), ('ST', 'Sao Tome and Principe'), ('SA', 'Saudi Arabia'), ('SN', 'Senegal'), ('RS', 'Serbia'), ('SC', 'Seychelles'), ('SL', 'Sierra Leone'), ('SG', 'Singapore'), ('SX', 'Sint Maarten (Dutch part)'), ('SK', 'Slovakia'), ('SI', 'Slovenia'), ('SB', 'Solomon Islands'), ('SO', 'Somalia'), ('ZA', 'South Africa'), ('GS', 'South Georgia and the South Sandwich Islands'), ('SS', 'South Sudan'), ('ES', 'Spain'), ('LK', 'Sri Lanka'), ('SD', 'Sudan'), ('SR', 'Suriname'), ('SJ', 'Svalbard and Jan Mayen'), ('SZ', 'Swaziland'), ('SE', 'Sweden'), ('CH', 'Switzerland'), ('SY', 'Syrian Arab Republic'), ('TW', 'Taiwan (Province of China)'), ('TJ', 'Tajikistan'), ('TZ', 'Tanzania, United Republic of'), ('TH', 'Thailand'), ('TL', 'Timor-Leste'), ('TG', 'Togo'), ('TK', 'Tokelau'), ('TO', 'Tonga'), ('TT', 'Trinidad and Tobago'), ('TN', 'Tunisia'), ('TR', 'Turkey'), ('TM', 'Turkmenistan'), ('TC', 'Turks and Caicos Islands'), ('TV', 'Tuvalu'), ('UG', 'Uganda'), ('UA', 'Ukraine'), ('AE', 'United Arab Emirates'), ('GB', 'United Kingdom'), ('US', 'United States'), ('UM', 'United States Minor Outlying Islands'), ('UY', 'Uruguay'), ('UZ', 'Uzbekistan'), ('VU', 'Vanuatu'), ('VE', 'Venezuela, Bolivarian Republic of'), ('VN', 'Viet Nam'), ('VG', 'Virgin Islands (British)'), ('VI', 'Virgin Islands (U.S.)'), ('WF', 'Wallis and Futuna'), ('EH', 'Western Sahara'), ('YE', 'Yemen'), ('ZM', 'Zambia'), ('ZW', 'Zimbabwe'), ('AX', '\xc5land Islands')], max_length=2, blank=True, null=True, verbose_name='shipping country')),
                ('delivery_method', models.CharField(default=b'PERSONAL_PICKUP', max_length=64, verbose_name='delivery method', choices=[(b'PERSONAL_PICKUP', 'personal pickup'), (b'MAILING', 'mailing')])),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', models.DateTimeField(auto_now=True, verbose_name='modified')),
            ],
            options={
                'ordering': ('date_issue', 'number'),
                'db_table': 'invoicing_invoices',
                'verbose_name': 'invoice',
                'verbose_name_plural': 'invoices',
            },
        ),
        migrations.CreateModel(
            name='Item',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='title')),
                ('quantity', models.DecimalField(default=1, verbose_name='quantity', max_digits=10, decimal_places=3)),
                ('unit', models.CharField(default=b'PIECES', max_length=64, verbose_name='unit', choices=[(b'EMPTY', b''), (b'PIECES', 'pcs.'), (b'HOURS', 'hours')])),
                ('unit_price', models.DecimalField(verbose_name='unit price', max_digits=10, decimal_places=2)),
                ('tax_rate', models.DecimalField(decimal_places=1, default=None, max_digits=3, blank=True, help_text='VAT rate', null=True, verbose_name='tax rate (%)')),
                ('tag', models.CharField(default=None, max_length=128, null=True, verbose_name='tag', blank=True)),
                ('weight', models.IntegerField(default=0, choices=[(0, 0), (1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8), (9, 9), (10, 10), (11, 11), (12, 12), (13, 13), (14, 14), (15, 15), (16, 16), (17, 17), (18, 18), (19, 19)], blank=True, help_text='order', null=True, verbose_name='weight')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', models.DateTimeField(auto_now=True, verbose_name='modified')),
                ('invoice', models.ForeignKey(verbose_name='invoice', to='invoicing.Invoice', on_delete=models.CASCADE)),
            ],
            options={
                'ordering': ('-invoice', 'weight', 'created'),
                'db_table': 'invoicing_items',
                'verbose_name': 'item',
                'verbose_name_plural': 'items',
            },
        ),
    ]



import datetime

from django.db import connection
from django.db.models import Q, Count
from django.db.models.query import QuerySet
from django.utils.timezone import now


class InvoiceQuerySet(QuerySet):
    def overdue(self):
        return self.unpaid() \
            .filter(date_due__lt=datetime.datetime.combine(now().date(), datetime.time.max)) \
            .exclude(type=self.model.TYPE.CREDIT_NOTE)

    def not_overdue(self):
        return self.filter(Q(date_due__gt=datetime.datetime.combine(now().date(), datetime.time.max)) | Q(status__in=[self.model.STATUS.PAID, self.model.STATUS.CANCELED, self.model.STATUS.CREDITED]))

    def paid(self):
        return self.filter(status=self.model.STATUS.PAID)

    def unpaid(self):
        return self.exclude(Q(status__in=[self.model.STATUS.PAID, self.model.STATUS.CANCELED, self.model.STATUS.CREDITED]) | Q(total=0))

    def valid(self):
        return self.exclude(status__in=[self.model.STATUS.RETURNED, self.model.STATUS.CANCELED, self.model.STATUS.CREDITED, self.model.STATUS.UNCOLLECTIBLE])

    def collectible(self):
        return self.exclude(status=self.model.STATUS.UNCOLLECTIBLE)

    def uncollectible(self):
        return self.filter(status=self.model.STATUS.UNCOLLECTIBLE)

    def having_related_invoices(self):
        return self.exclude(related_invoices=None)

    def not_having_related_invoices(self):
        return self.filter(related_invoices=None)

    def duplicate_numbers(self):
        return list(self
                    .values('number')
                    .order_by()
                    .annotate(count=Count('id'))
                    .filter(count__gt=1)
                    .values_list('number', flat=True))

    def lock(self):
        """ Lock table.

        Locks the object model table so that atomic update is possible.
        Simultaneous database access request pend until the lock is unlock()'ed.

        Note: If you need to lock multiple tables, you need to do lock them
        all in one SQL clause and this function is not enough. To avoid
        dead lock, all tables must be locked in the same order.

        If no lock mode is specified, then ACCESS EXCLUSIVE, the most restrictive mode, is used.
        Read more: https://www.postgresql.org/docs/9.4/static/sql-lock.html
        """
        cursor = connection.cursor()
        table = self.model._meta.db_table
        cursor.execute("LOCK TABLE %s" % table)


class ItemQuerySet(QuerySet):
    def with_tag(self, tag):
        return self.filter(tag=tag)
